from links_pydriver.connection import db_utils
from links_pydriver.model.Experiment import Experiment
from links_pydriver.model.Relation import Relation
from links_pydriver.model.Snapshot import Snapshot
from links_pydriver.model.Entity import Entity
from links_pydriver.marshaler.DriverMarshaler import marshalling


def create_experience():
    name = "exp_example"
    attribute_map = {'Description': 'Example links data structure',
                     'Configuration': {'confName': 'MyConf1',
                                       'confProfile': 'DEBUG'},
                     'Version': '0.1'}

    exp = Experiment(name, attribute_map, None)
    return exp


def create_entities():
    e1 = Entity("Agent 1", "Ant", {'role': 'Furrowmaker', 'name': 'Maholise', 'health': 30})
    e2 = Entity("Agent 2", "Ant", {'role': 'Furrowmaker', 'name': 'Tuoi', 'health': 90})
    e3 = Entity("Agent 3", "Ant", {'role': 'Sheperdess', 'name': 'Loumwe', 'health': 50})
    e4 = Entity("Agent 4", "Ant", {'role': 'Warrior', 'name': 'Ereat', 'health': 15})
    e5 = Entity("Agent 5", "Termite", {'role': 'Warrior', 'name': 'Yherzok', 'health': 2})
    l_entities = [e1, e2, e3, e4, e5]

    return l_entities


def create_relations():
    r1 = Relation('helping1', 'Agent 2', 'Agent 1', True, 'Help')
    r2 = Relation('helping1', 'Agent 3', 'Agent 1', True, 'Help')
    r3 = Relation('fight1', 'Agent 4', 'Agent 5', False, 'Fight')
    l_relations = [r1, r2, r3]

    return l_relations


if __name__ == '__main__':
    db = db_utils.connect()
    exp = create_experience()
    list_entities = create_entities()
    list_relations = create_relations()

    s1 = Snapshot(0, list_entities[:4], list_relations[:2], {'Encontred exception': 'Unhandle NCS !'})
    s2 = Snapshot(1, list_entities, list_relations, {'Encontred exception': 'Unhandle NCS !'})

    list_snapshots = [s1, s2]

    exp.snapshots = list_snapshots

    marshalling(db, exp)


class Entity():
    def __init__(self, entityID, type, attribute_map):
        self.__entityID = entityID
        self.__type = type
        self.__attribute_map = attribute_map

    @property
    def entityID(self):
        return self.__entityID

    @entityID.setter
    def entityID(self, entityID):
        self.__entityID = entityID

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, type):
        self.__type = type

    @property
    def attribute_map(self):
        return self.__attribute_map

    @attribute_map.setter
    def attribute_map(self, attribute_map):
        self.__attribute_map = attribute_map




class Snapshot:
    def __init__(self, snapshot_number, entities, relations, attribute_map):
        self.__snapshot_number = snapshot_number
        self.__entities = entities
        self.__relations = relations
        self.__attribute_map = attribute_map

    @property
    def snapshot_number(self):
        return self.__snapshot_number

    @snapshot_number.setter
    def snapshot_number(self, snapshot_number):
        self.__snapshot_number = snapshot_number

    @property
    def entities(self):
        return self.__entities

    @entities.setter
    def entities(self, entities):
        self.__entities = entities

    @property
    def relations(self):
        return self.__relations

    @relations.setter
    def relations(self, relations):
        self.__relations = relations

    @property
    def attribute_map(self):
        return self.__attribute_map

    @attribute_map.setter
    def attribute_map(self, attribute_map):
        self.__attribute_map = attribute_map



class Relation:
    def __init__(self, relationID, nodeAid, nodeBid, is_oriented, type):
        self.__relationID = relationID
        self.__nodeAid = nodeAid
        self.__nodeBid = nodeBid
        self.__is_oriented = is_oriented
        self.__type = type

    @property
    def relationID(self):
        return self.__relationID

    @relationID.setter
    def relationID(self, relationID):
        self.__relationID = relationID

    @property
    def nodeAid(self):
        return self.__nodeAid

    @nodeAid.setter
    def nodeAid(self, nodeAid):
        self.__nodeAid = nodeAid

    @property
    def nodeBid(self):
        return self.__nodeBid

    @nodeBid.setter
    def nodeBid(self, nodeBid):
        self.__nodeBid = nodeBid

    @property
    def is_oriented(self):
        return self.__is_oriented

    @is_oriented.setter
    def is_oriented(self, is_oriented):
        self.__is_oriented = is_oriented

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, type):
        self.__type = type



class Experiment:
    def __init__(self, name, attribute_map, snapshots):
        self.__name = name
        self.__attribute_map = attribute_map
        self.__snapshots = snapshots

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def attribute_map(self):
        return self.__attribute_map

    @attribute_map.setter
    def attribute_map(self, attribute_map):
        self.__attribute_map = attribute_map

    @property
    def snapshots(self):
        return self.__snapshots

    @snapshots.setter
    def snapshots(self, snapshots):
        self.__snapshots = snapshots



from setuptools import setup, find_packages

setup(name='links_pydriver',
      version='0.2',
      description='A python driver for Links',
      url='https://github.com/tanguyesteoule/links_pydriver',
      author='Tanguy Esteoule',
      author_email='tanguy.esteoule@gmail.com',
      license='MIT',
      packages = find_packages(),
      zip_safe=False)